<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Category;
use Illuminate\Http\Request;

class CategoriesController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $rows = Category::whereNull('parent_id')->orderByDesc('id')->get();
        return view('admin.categories.index', compact('rows'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.categories.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'name_ar' => 'required',
            'name_en' => 'required',
            'image' => 'required|image',
            'status' => 'required|in:active,not_active',
            'car_option' => 'required|in:true,false',
        ]);
        $category = new Category();
        $category['name->ar'] = $request['name_ar'];
        $category['name->en'] = $request['name_en'];
        $category->setImageAttribute();
        $category['status'] = $request['status'];
        $category['car_option'] = $request['car_option'];
        $category->save();
        return redirect()->route('categories.index')->with('message', 'تم الإنشاء بنجاح');
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $category = Category::find($id);

//        if (isset($category->parent)){
//            dd('true');
//        }else{
//            dd('false');
//        }


        $rows = Category::where('parent_id', $id)->orderbyDesc('id')->get();
        return view('admin.categories.show', compact('category', 'rows'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $row = Category::find($id);
        return view('admin.categories.edit', compact('row'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'name_ar' => 'required',
            'name_en' => 'required',
            'image' => 'image',
            'status' => 'required|in:active,not_active',
            'car_option' => 'required|in:true,false',
        ]);
        $category = Category::find($id);
        $category['name->ar'] = $request['name_ar'];
        $category['name->en'] = $request['name_en'];
        if ($request->hasFile('image')) $category->setImageAttribute();
        $category['status'] = $request['status'];
        $category['car_option'] = $request['car_option'];
        $category->update();
        return redirect()->route('categories.index')->with('message', 'تم التعديل بنجاح');

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $row = Category::find($id);

        $row->deleteImage();

        Category::destroy($id);
        return redirect()->back()->with('message', 'تم الحذف بنجاح');
    }
}
