<?php

namespace App\Http\Controllers\web;

use App\Http\Controllers\Controller;
use App\Http\Resources\CommentsResource;
use App\Http\Resources\ProductsResource;
use App\Models\AdditionalType;
use App\Models\Category;
use App\Models\City;
use App\Models\Comment;
use App\Models\Favourite;
use App\Models\Follower;
use App\Models\Notification;
use App\Models\Product;
use App\Models\ProductImage;
use App\Models\Spam;
use App\Models\UserToken;
use App\Models\View;
use Carbon\Carbon;
use Edujugon\PushNotification\PushNotification;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Lang;

class ProductsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
    }

    public function car_option_rules()
    {
        return [
            'title' => 'required',
            'grand_id' => 'required|exists:categories,id',
            'city_id' => 'required|exists:cities,id',
            'county_id' => 'required_if:city_id,null|exists:cities,id',
            'hide_mobile' => 'in:true,false',
            'mobile' => 'required_if:hide_mobile,false',
            'country_code' => 'required_if:hide_mobile,false',
            'ad_type' => 'required|in:buy,waiver',
            'year' => 'required',
            'car_status' => 'required|in:used,new,accident',
            'fetis' => 'required|in:manual,automatic',
            'fuel' => 'required|in:gasoline,hybrid,diesel',
            'double' => 'required|in:true,false',
            'km' => 'required',
            'detail' => 'required',
            'lat' => 'nullable',
            'lng' => 'nullable',
            'installment_destination' => 'nullable',
            'determine_price' => 'required|in:true,false',
            'price' => 'required_if:determine_price,true',
            'swear' => 'accepted',
            'images' => 'required'
        ];
    }

    public function rules()
    {
        return [
            'title' => 'required',
            'grand_id' => 'required|exists:categories,id',
            'city_id' => 'required|exists:cities,id',
            'county_id' => 'required_if:city_id,null|exists:cities,id',
            'hide_mobile' => 'in:true,false',
            'mobile' => 'required_if:hide_mobile,false',
            'country_code' => 'required_if:hide_mobile,false',

            'ad_type' => 'nullable|in:buy,waiver',
            'year' => 'nullable',
            'car_status' => 'nullable|in:used,new,accident',
            'fetis' => 'nullable|in:manual,automatic',
            'fuel' => 'nullable|in:gasoline,hybrid,diesel',
            'double' => 'nullable|in:true,false',
            'km' => 'nullable',
            'detail' => 'required',
            'lat' => 'nullable',
            'lng' => 'nullable',
            'installment_destination' => 'nullable',
            'determine_price' => 'required|in:true,false',
            'price' => 'required_if:determine_price,true',
            'swear' => 'accepted',
            'images' => 'required'


        ];
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        if (!\Auth::check()) return redirect()->route('home')->with('error', \Lang::get('data.login_first'));

        $ad_types = AdditionalType::all();
        $countries = City::where('type', 'country')->where('parent_id', null)->get();
        $categories = Category::where('parent_id', null)->get();


        return view('web.products.create', compact('ad_types', 'countries', 'categories'));

    }

    public function refresh_product(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'product_code' => 'required'
        ]);
        if ($validator->passes()) {


            $product = Product::whereCode($request['product_code'])->first();
            if ($product->updates_number < 2 && Carbon::now()->toDateString() == Carbon::parse($product->created_at)->toDateString())
            {

                $product->created_at = Carbon::now();
                $product->updated_at = Carbon::now();
                $product->updates_number += 1;
                $product->update();
                return response()->json(['value' => true,'msg'=>\Lang::get('data.refresh_success'), 'code' => 200]);

            } elseif (Carbon::now()->toDateString() != Carbon::parse($product->created_at)->toDateString()) {
                $product->created_at = Carbon::now();
                $product->updated_at = Carbon::now();
                $product->updates_number = 1;
                $product->update();
                return response()->json(['value' => true, 'msg'=>\Lang::get('data.refresh_success'), 'code' => 200]);

            } else {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.cant_refresh'), 'code' => 422]);
            }


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function grand_categories_ajax(Request $request)
    {
        $ad_type = AdditionalType::find($request->ad_type); // Ad type from request


        $categories = Category::whereParentId(null)->get(); // all grand without parent id


        $single_category = Category::find($ad_type->category_id); // exact category

        if ($single_category->car_option == 'true') {
            $car_option = 'true';
        } else {
            $car_option = 'false';
        }

        $parent_categories = Category::whereParentId($single_category->id)->get();
        $grand_view = view('web.products.grand_category_foreach', compact('ad_type', 'categories', 'single_category'))->render();
        $parent_view = view('web.products.parent_category_foreach', compact('parent_categories'))->render();
        return response()->json(['value' => true, 'grand_view' => $grand_view, 'parent_view' => $parent_view, 'car_option' => $car_option]);
    }

    public function get_categories_ajax(Request $request)
    {
        $selected_category = Category::find($request->category_id);
        if (!$selected_category) return response()->json(['display' => 'none']);

        if ($selected_category->car_option == 'true') {
            $car_option = 'true';
        } else {
            $car_option = 'false';
        }
        $child_categories = Category::whereParentId($request->category_id)->get();
        if (count($child_categories) < 1) {
            $display = 'none';
        } else {
            $display = 'block';

        }
        $child_view = view('web.products.categories_foreach', compact('child_categories'))->render();
        return response()->json(['value' => true, 'child_view' => $child_view, 'car_option' => $car_option, 'display' => $display]);
    }

    public function get_parent_categories_ajax(Request $request)
    {

        $selected_category = Category::find($request->category_id);
        if (!$selected_category) return response()->json(['display' => 'none']);

        if ($selected_category->car_option == 'true') {
            $car_option = 'true';
        } else {
            $car_option = 'false';
        }
        $parent_categories = Category::whereParentId($request->category_id)->get();

        if (count($parent_categories) < 1) {
            $display = 'none';
        } else {
            $display = 'block';
        }

        $parent_view = view('web.products.parent_category_foreach', compact('parent_categories'))->render();
        return response()->json(['value' => true, 'parent_view' => $parent_view, 'car_option' => $car_option, 'parent_categories' => $parent_categories, 'display' => $display]);
    }

    public function get_cities_ajax(Request $request)
    {
        $cities = City::whereParentId($request->city_id)->get();
        $cities_view = view('web.products.cities_foreach', compact('cities'))->render();
        if (count($cities) < 1) {
            $display = 'none';
        } else {
            $display = 'block';
        }
        return response()->json(['value' => true, 'cities_view' => $cities_view, 'display' => $display]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
//        return  $request->all();
        if (!\Auth::check()) return redirect()->back()->with('error', \Lang::get('data.login_first'));


        \Session::put('ads_type_session', $request['ads_type']);
        \Session::put('city_id_session', $request['city_id']);
        \Session::put('county_id_session', $request['county_id']);
        \Session::put('grand_id_session', $request['grand_id']);
        \Session::put('parent_id_session', $request['parent_id']);
        \Session::put('category_id_session', $request['category_id']);

        $user = \Auth::user();

        if ($user->vip == 'false') {
            $limit = 2;
            $last_products = Product::where(['status' => 'active', 'user_id' => $user->id])->orderByDesc('id')->take($limit)->pluck('created_at');
        } else {
            $limit = 4;
            $last_products = Product::where(['status' => 'active', 'user_id' => $user->id])->orderByDesc('id')->take($limit)->pluck('created_at');
        }
        $count_data = 0;
        foreach ($last_products as $last_product) {
            if (Carbon::now()->toDateString() == Carbon::parse($last_product)->toDateString()) {
                $count_data += 1;
            }
        }
        if ($count_data >= $limit) {
//            return response()->json(['value' => false, 'msg' => Lang::get('data.check_add')]);
            return redirect()->back()->with('error', \Lang::get('data.check_add'));
        }


        $grand_id = $request['grand_id'];
        $parent_id = $request['parent_id'];
        $category_id = $request['category_id'];

        // grand
        // parent
        // category is child

        if ($grand_id != null && $parent_id == null && $category_id == null) {

            $final_category_id = $grand_id;

        } elseif ($grand_id != null && $parent_id != null && $category_id == null) {
            $final_category_id = $parent_id;
        } elseif ($grand_id != null && $parent_id != null && $category_id != null) {
            $final_category_id = $category_id;
        } else {
            return redirect()->back()->with('error', \Lang::get('data.category_missed'));

        }


        $category = Category::find($final_category_id);
        if ($category->car_option == 'true') {

            $this->validate($request, [
                'title' => 'required',
                'grand_id' => 'required|exists:categories,id',
                'city_id' => 'required|exists:cities,id',
                'county_id' => 'required_if:city_id,null|exists:cities,id',
                'hide_mobile' => 'in:true,false',
                'mobile' => 'required_if:hide_mobile,false',
                'country_code' => 'required_if:hide_mobile,false',
                'ad_type' => 'required|in:buy,waiver',
                'year' => 'required',
                'car_status' => 'required|in:used,new,accident',
                'fetis' => 'required|in:manual,automatic',
                'fuel' => 'required|in:gasoline,hybrid,diesel',
                'double' => 'required|in:true,false',
                'km' => 'required',
                'detail' => 'required',
                'lat' => 'nullable',
                'lng' => 'nullable',
                'installment_destination' => 'nullable',
                'determine_price' => 'required|in:true,false',
                'price' => 'required_if:determine_price,true',
                'swear' => 'accepted',
                'images' => 'required'
            ], [
                'title.required' => \Lang::get('data.title_required'),
                'grand_id.required' => \Lang::get('data.grand_id_required'),
                'city_id.required' => \Lang::get('data.city_id_required'),
                'city_id.exists' => \Lang::get('data.city_id_required'),
                'county_id.required_if' => \Lang::get('data.city_id_required'),

                'mobile.required_if' => \Lang::get('data.mobile_required_if'),
                'country_code.required_if' => \Lang::get('data.country_code_required_if'),
                'ad_type.required' => \Lang::get('data.ad_type_required'),
                'year.required' => \Lang::get('data.year_required'),
                'car_status.required' => \Lang::get('data.car_status_required'),
                'fetis.required' => \Lang::get('data.fetis_required'),
                'fuel.required' => \Lang::get('data.fuel_required'),
                'double.required' => \Lang::get('data.double_required'),
                'km.required' => \Lang::get('data.km_required'),
                'detail.required' => \Lang::get('data.detail_required'),
                'determine_price.required' => \Lang::get('data.determine_price_required'),
                'price.required_if' => \Lang::get('data.price_required_if'),
                'swear.accepted' => \Lang::get('data.swear_accepted'),
                'images.required' => \Lang::get('data.images_required'),

            ]);
        } else {

            $this->validate($request, [
                'title' => 'required',
                'grand_id' => 'required|exists:categories,id',
                'city_id' => 'required|exists:cities,id',
                'county_id' => 'required_if:city_id,null|exists:cities,id',
                'hide_mobile' => 'in:true,false',
                'mobile' => 'required_if:hide_mobile,false',
                'country_code' => 'required_if:hide_mobile,false',
                'ad_type' => 'nullable|in:buy,waiver',
                'year' => 'nullable',
                'car_status' => 'nullable|in:used,new,accident',
                'fetis' => 'nullable|in:manual,automatic',
                'fuel' => 'nullable|in:gasoline,hybrid,diesel',
                'double' => 'nullable|in:true,false',
                'km' => 'nullable',
                'detail' => 'required',
                'lat' => 'nullable',
                'lng' => 'nullable',
                'installment_destination' => 'nullable',
                'determine_price' => 'required|in:true,false',
                'price' => 'required_if:determine_price,true',
                'swear' => 'accepted',
                'images' => 'required'


            ], [
                'title.required' => \Lang::get('data.title_required'),
                'grand_id.required' => \Lang::get('data.grand_id_required'),
                'city_id.required' => \Lang::get('data.city_id_required'),
                'city_id.exists' => \Lang::get('data.city_id_required'),
                'county_id.required_if' => \Lang::get('data.city_id_required'),

                'mobile.required_if' => \Lang::get('data.mobile_required_if'),
                'country_code.required_if' => \Lang::get('data.country_code_required_if'),
                'detail.required' => \Lang::get('data.detail_required'),
                'determine_price.required' => \Lang::get('data.determine_price_required'),
                'price.required_if' => \Lang::get('data.price_required_if'),
                'swear.accepted' => \Lang::get('data.swear_accepted'),
                'images.required' => \Lang::get('data.images_required')
            ]);
        }

        $mobile = $request['country_code'] . $request['mobile'];
        $user = \Auth::user();
        $product = new Product();
        $product['user_id'] = $user->id;
        $product['title'] = $request['title'];
        $product['category_id'] = $category->id;
        $product['city_id'] = $request['county_id'] ? $request['county_id'] : $request['city_id'];
        $product['mobile'] = $mobile;
        $product['hide_mobile'] = $request['hide_mobile'];
        $product['ad_type'] = $request['ad_type'];
        $product['year'] = $request['year'];
        $product['car_status'] = $request['car_status'];
        $product['installment_destination'] = $request['installment_destination'];
        $product['fetis'] = $request['fetis'];
        $product['fuel'] = $request['fuel'];
        $product['double'] = $request['double'];
        $product['km'] = $request['km'];
        $product['determine_price'] = $request['determine_price'];
        $product['price'] = $request['price'];
        $product['detail'] = $request['detail'];
        $product['lat'] = $request['lat'];
        $product['lng'] = $request['lng'];
        $product['views'] = 0;
        $product['status'] = 'active';
        $product['code'] = uniqid();
        $product->save();

        if ($request->images) {
            $image_array = $request['images'];
            $i = 1;
            foreach ($image_array as $image) {
                $filename = time() . $i . '.' . $image->getClientOriginalExtension();
                $image->move(public_path('images/products'), $filename);
                ProductImage::create([
                    'product_id' => $product->id,
                    'image' => $filename
                ]);
                $i++;
            }
        }

        if (count($product->images) == 0) {

            $notification = new Notification();
            $notification['user_id'] = $user->id;
            $notification['body->ar'] = 'عميلنا العزيز اضافه صوره لأعلانكم يزيد من نسبه التفاعل ';
            $notification['body->en'] = 'Our dear customer, adding a picture to your advertisement increases the rate of interaction';
            $notification['title->en'] = 'Add Images';
            $notification['product_id'] = $product->id;
            $notification['title->ar'] = 'إضافة صور';
            $notification['type'] = 'admin';
            $notification->save();

            $all_tokens = UserToken::whereStatus('connected')->whereUserId($user->id)->get();
            $push = new PushNotification('fcm');

            foreach ($all_tokens as $token) {

                if ($token->device_type == 'ios') {
                    $msg = [
                        'data' => [
                            'title_ar' => $notification->title['ar'],
                            'title_en' => $notification->title['en'],
                            'body_ar' => $notification->body['ar'],
                            'body_en' => $notification->body['en'],
                            'type' => $notification->type,
                            'product_id' => $product->id,
                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'notification' => [
                            'title_ar' => $notification->title['ar'],
                            'title_en' => $notification->title['en'],
                            'body_ar' => $notification->body['ar'],
                            'body_en' => $notification->body['en'],
                            'type' => $notification->type,
                            'product_id' => $product->id,

                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token)
                        ->send();

                } else {
                    $msg = [
                        'data' => [
                            'title_ar' => $notification->title['ar'],
                            'title_en' => $notification->title['en'],
                            'body_ar' => $notification->body['ar'],
                            'body_en' => $notification->body['en'],
                            'type' => $notification->type,
                            'product_id' => $product->id,

                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token)
                        ->send();

                }


            }


        }


        $user_name = $user->name ? $user->name : $user->mobile;
        $followers = $user->followers->pluck('id');
        $followers_tokens = UserToken::whereIn('user_id', $followers)->whereStatus('connected')->get();
        $push = new PushNotification('fcm');

        foreach ($followers_tokens as $token) {

            if ($token->device_type == 'ios') {
                $msg = [
                    'data' => [
                        'title_ar' => " قام  $user_name بإنشاء إعلان جديد  ",
                        'title_en' => "Your Following $user_name posted a new product",
                        'body_ar' => " قام  $user_name بإنشاء إعلان جديد  ",
                        'body_en' => "Your Following $user_name posted a new product",
                        'type' => "follower",
                        'product_id' => $product->id,
                        'sound' => 'default',
                        'click_action' => 'FCM_PLUGIN_ACTIVITY',
                    ],
                    'notification' => [
                        'title_ar' => " قام  $user_name بإنشاء إعلان جديد  ",
                        'title_en' => "Your Following $user_name posted a new product",
                        'body_ar' => " قام  $user_name بإنشاء إعلان جديد  ",
                        'body_en' => "Your Following $user_name posted a new product",
                        'type' => "follower",
                        'product_id' => $product->id,

                        'sound' => 'default',
                        'click_action' => 'FCM_PLUGIN_ACTIVITY',
                    ],
                    'priority' => 'high',
                ];
                $push->setMessage($msg)
                    ->setDevicesToken($token)
                    ->send();

            } else {
                $msg = [
                    'data' => [
                        'title_ar' => " قام  $user_name بإنشاء إعلان جديد  ",
                        'title_en' => "Your Following $user_name posted a new product",
                        'body_ar' => " قام  $user_name بإنشاء إعلان جديد  ",
                        'body_en' => "Your Following $user_name posted a new product",
                        'type' => "follower",
                        'product_id' => $product->id,

                        'sound' => 'default',
                        'click_action' => 'FCM_PLUGIN_ACTIVITY',
                    ],
                    'priority' => 'high',
                ];
                $push->setMessage($msg)
                    ->setDevicesToken($token)
                    ->send();

            }

            $notification = new Notification();
            $notification['user_id'] = $token->user_id;
            $notification['body->ar'] = " قام  $user_name بإنشاء إعلان جديد  ";
            $notification['body->en'] = "Your Following $user_name posted a new product";
            $notification['title->en'] = "Your Following $user_name posted a new product";
            $notification['product_id'] = $product->id;
            $notification['title->ar'] = " قام  $user_name بإنشاء إعلان جديد  ";
            $notification['type'] = 'follower';
            $notification->save();

        }


        return redirect()->route('product.show', $product->code)->with('message', \Lang::get('data.product_added'));


    }

    /**
     * Display the specified resource.
     *
     * @param int $code
     * @return \Illuminate\Http\Response
     */
    public function show($code)
    {
        $row = Product::whereCode($code)->first();
        $follow_status = 'false';
        if (\Auth::check()) {
            $user = \Auth::user();
            $check = View::where(['user_id' => $user->id, 'product_id' => $row->id])->first();
            if (!$check) {
                View::create([
                    'user_id' => $user->id,
                    'product_id' => $row->id
                ]);
                $product = Product::find($row->id);
                $product->views += 1;
                $product->update();
            }

            $follow_check = Follower::where(['user_id' => $user->id, 'product_id' => $row->id])->first();
            if ($follow_check) {
                $follow_status = 'true';
            } else {
                $follow_status = 'false';
            }

        }

//        \Session::flush();
        $similar_products = Product::where('category_id', $row->category->id)->where('id', '!=', $row->id)->take(6)->orderByDesc('id')->get();
        return view('web.products.show', compact('row', 'similar_products', 'follow_status'));
    }

    public function follow_product(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'product_code' => 'required|exists:products,code'
        ]);
        if ($validator->passes()) {
            $product = Product::whereCode($request['product_code'])->first();
            $follow_check = Follower::where(['user_id' => \Auth::user()->id, 'product_id' => $product->id])->first();
            if ($follow_check) {
                $follow_check->delete();
                $follow_status = \Lang::get('data.follow_comments');
                $msg = \Lang::get('data.unfollow_success');
            } else {
                Follower::create(['user_id' => \Auth::user()->id, 'product_id' => $product->id]);
                $follow_status = \Lang::get('data.unfollow_comments');
                $msg = \Lang::get('data.follow_success');
            }
            return response()->json(['value' => true, 'msg' => $msg, 'follow_status' => $follow_status]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }

    public function add_comment_web(Request $request)
    {
        if ($request->comment) {
            $row = new Comment();
            $row->product_id = $request->product_id;
            $row->user_id = \Auth::user()->id;
            $row->body = $request->comment;
            $row->save();
            return response()->json(['value' => true, 'msg' => \Lang::get('data.add_comment'), 'data' => new CommentsResource($row)]);
        }
        return response()->json(['value' => false, 'msg' => \Lang::get('data.error_comment'), 'data' => []]);

    }

    public function spam_web(Request $request)
    {
        $spam = Spam::where(['user_id' => \Auth::user()->id, 'product_id' => $request->product_id, 'type' => $request->type, 'comment_id' => $request->comment_id])->first();
        if (!$spam) {
            $row = new Spam();
            $row->user_id = \Auth::user()->id;
            $row->product_id = $request->product_id;
            $row->type = $request->type;
            if ($request->type == 'comment') {
                $row->comment_id = $request->comment_id;
            }
            $row->save();
            return response()->json(['msg' => \Lang::get('data.spam_web')]);
        }
    }

    public function fav_product(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'row_id' => 'required'
        ]);
        if ($validator->passes()) {

            if (!\Auth::check()) return response()->json(['value' => false, 'msg' => \Lang::get('data.login_first')]);
            $check = Favourite::where(['user_id' => Auth::user()->id, 'product_id' => $request['row_id']])->first();
            if ($check) {
                $check->delete();
                $url = asset('web/img/dynamic/icons/heart.png');
                $msg = \Lang::get('data.unlike');
            } else {
                Favourite::create(['user_id' => Auth::user()->id, 'product_id' => $request['row_id']]);
                $msg = \Lang::get('data.liked');
                $url = asset('web/img/dynamic/icons/red_heart.png');

            }
            $new_count = Favourite::where('product_id', $request['row_id'])->count();

            return response()->json(['value' => true, 'msg' => $msg, 'new_count' => $new_count, 'url' => $url]);


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }
}
